unit delays;                                    { Delay routines, v28   }
interface                                       { By Jens Dyekjr Madsen}
{$s-}
const
  t2u5 = -5;                                    { (5+1)*419ns = 2.51s  }
  t5u  = -11;                                   { 12 * 419ns  = 5.03s  }
  t7u5 = -17;                                   { 18 * 419ns  = 7.54s  }
  t10u = -23;                                   { 24 * 419ns  = 10.06s }
  sdt  = t7u5;                                  { Shortdelay  = 7.5s   }

procedure shortdelay;                           { Short delay           }
inline(
  $E4/$42/      {       IN      AL,[42]         ; Read timer 2, LSB     }
  $B3/<sdt/     {       MOV     BL,sdt          ; Setup delay time      }
  $00/$C3/      {       ADD     BL,AL           ; Calculate stop time   }
  $EB/$00/      {       JMP     $+2             ; I/O delay             }
  $EB/$00/      {L1:    JMP     $+2             ; I/O delay             }
  $E4/$42/      {       IN      AL,[42]         ; Read timer 2, LSB     }
  $38/$D8/      {       CMP     AL,BL           ; Compare with stoptime }
  $79/$F8);     {       JNS     L1              ; Continue until timeout}

function  shortdelay_err: boolean;              { Test if timer 2 free? }
procedure shortdelay_on;                        { Start timer 2, cli    }
procedure shortdelay_off;                       { Stop timer 2, sti     }


implementation

procedure cli; inline($fa);                     { Disable interrupts    }
procedure sti; inline($fb);                     { Enable interrupts     }
procedure pushf; inline($9c);                   { Save flags on stack   }
procedure popf; inline($9d);                    { Get flags from stack  }
procedure nop; inline($90);                     { No operation          }

var
  gsave: byte;                                  { Saves 8255, portB     }

function shortdelay_err: boolean;               { Timer 2 free?         }
begin
  shortdelay_err:=((port[$61] and 3)<>0);       { Check if timer 2 free }
end;

procedure shortdelay_on;                        { Start timer           }
begin
  repeat                                        { Wait on timer 2 free  }
    sti; nop;                                   { Allow interrupts      }
    cli;                                        { CLI when timer in use }
    gsave:=port[$61];                           { Store 8255 port B     }
  until (gsave and 3) = 0;                      { Repeat to timer 2 free}
  port[$43]:=$96;                               { R/W LSB only          }
  port[$61]:=gsave or 1;                        { Timer 2 gate on       }
  port[$42]:=0;                                 { Begin count on timer 2}
end;

procedure shortdelay_off;                       { Stop timer            }
begin
  port[$61]:=gsave;                             { Stop timer 2          }
  sti;                                          { Interrupts on         }
end;

end.
